import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Spin, notification, Modal, Button } from 'antd'

import Api from '@/api'
import UtilsDM from '@/utils/utils-datamanage.js'
import asyncComponent from '@/utils/asyncComponent'
import asyncSpinComponent from '@/utils/asyncSpinComponent'
import MKEmitter from '@/utils/events.js'
import './index.scss'

const MutilForm = asyncSpinComponent(() => import('@/tabviews/zshare/mutilform'))
const NormalButton = asyncComponent(() => import('@/tabviews/zshare/actionList/normalbutton'))
const NormalHeader = asyncComponent(() => import('@/tabviews/custom/components/share/normalheader'))

class SimpleForm extends Component {
  static propTpyes = {
    config: PropTypes.object
  }

  state = {
    BID: '',
    config: null,
    loading: false,
    data: null,
    group: null,
    BData: ''
  }

  UNSAFE_componentWillMount () {
    const { config } = this.props

    let _config = fromJS(config).toJS()
    let _data = null
    let BID = ''
    let BData = ''

    if (_config.setting.supModule) {
      BData = window.GLOB.CacheData.get(_config.setting.supModule)
    } else {
      BData = window.GLOB.CacheData.get(_config.$pageId)
    }
    if (BData) {
      BID = BData.$BID || ''
    }

    if (_config.wrap.datatype === 'dynamic') {
      _config.setting.onload = _config.setting.sync === 'true' ? 'false' : 'true'

      if (_config.setting.sync === 'true' && window.GLOB.SyncData.has(_config.dataName)) {
        _data = window.GLOB.SyncData.get(_config.dataName) || []
        _data = _data[0] || {$$empty: true}
        _data.$$uuid = _data[_config.setting.primaryKey] || ''

        _config.setting.sync = 'false'
  
        window.GLOB.SyncData.delete(_config.dataName)
      }
    } else if (_config.wrap.datatype === 'public') {
      if (window.GLOB.CacheData.has(_config.wrap.publicId)) {
        _data = window.GLOB.CacheData.get(_config.wrap.publicId)
        _data = fromJS(_data).toJS()
        _data.$$BID = BID || ''
        _data.$$BData = BData || ''
      }
    } else {
      _data = {$$empty: true}
    }

    let _group = _config.subcards[0]

    if (_group.subButton.enable === 'false' && (!_group.closeButton || _group.closeButton.enable !== 'true') && (!_group.resetButton || _group.resetButton.enable !== 'true')) {
      _group.subButton.style.display = 'none'
      _group.$button = 'no-button'
    }

    this.setState({
      data: _data,
      group: _group,
      BID: BID || '',
      BData: BData || '',
      config: _config
    })
  }

  componentDidMount () {
    const { config } = this.state

    MKEmitter.addListener('reloadData', this.reloadData)
    MKEmitter.addListener('mkFormSubmit', this.mkFormSubmit)
    MKEmitter.addListener('resetSelectLine', this.resetParentParam)
    MKEmitter.addListener('refreshByButtonResult', this.refreshByButtonResult)

    if (config.setting.sync === 'true') {
      MKEmitter.addListener('transferSyncData', this.transferSyncData)
    }

    if (config.wrap.datatype === 'public') {
      MKEmitter.addListener('mkPublicData', this.mkPublicData)
    } else if (config.setting.useMSearch) {
      MKEmitter.addListener('searchRefresh', this.searchRefresh)
    }

    this.initExec()
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('reloadData', this.reloadData)
    MKEmitter.removeListener('mkPublicData', this.mkPublicData)
    MKEmitter.removeListener('mkFormSubmit', this.mkFormSubmit)
    MKEmitter.removeListener('searchRefresh', this.searchRefresh)
    MKEmitter.removeListener('resetSelectLine', this.resetParentParam)
    MKEmitter.removeListener('transferSyncData', this.transferSyncData)
    MKEmitter.removeListener('refreshByButtonResult', this.refreshByButtonResult)
  }

  initExec = () => {
    const { config } = this.state
    
    if (config.wrap.datatype === 'dynamic' && config.setting.onload === 'true') {
      setTimeout(() => {
        this.loadData()
      }, config.setting.delay || 0)
    }
  }

  transferSyncData = (syncId) => {
    const { config } = this.state

    if (config.$syncId !== syncId) return

    let _data = window.GLOB.SyncData.get(config.dataName) || []
    _data = _data[0] || {$$empty: true}
    _data.$$uuid = _data[config.setting.primaryKey] || ''

    this.setState({data: _data})

    window.GLOB.SyncData.delete(config.dataName)

    MKEmitter.removeListener('transferSyncData', this.transferSyncData)
  }

  mkPublicData = (publicId, data) => {
    const { config, BID, BData } = this.state

    if (config.wrap.datatype === 'public' && config.wrap.publicId === publicId) {
      let _data = fromJS(data).toJS()

      _data.$$BID = BID || ''
      _data.$$BData = BData || ''

      this.setState({
        data: null
      }, () => {
        this.setState({data: _data})
      })
    }
  }

  searchRefresh = (searchId) => {
    const { config } = this.state

    if (config.$searchId !== searchId) return
    
    this.setState({}, () => {
      this.loadData()
    })
  }

  reloadData = (menuId, id) => {
    const { config, group } = this.state

    if (config.uuid !== menuId) return

    if (id === 'focus-refresh' && config.wrap.refocus) {
      let _group = fromJS(group).toJS()
      _group.setting.focus = config.wrap.refocus

      if (config.wrap.datatype === 'static') {
        this.setState({
          data: null,
          group: _group
        }, () => {
          this.setState({data: {$$empty: true}})
        })
      } else {
        this.setState({
          group: _group
        }, () => {
          this.loadData(true)
        })
      }
    } else if (id === 'focus-nofresh' && config.wrap.refocus) {
      let formId = ''
      group.fields.forEach(item => {
        if (item.field === config.wrap.refocus) {
          formId = item.uuid
        }
      })
      MKEmitter.emit('mkFC', 'focus', formId)
    } else {
      this.loadData()
    }
  }

  /**
   * @description 按钮执行完成后页面刷新
   * @param {*} menuId     // 菜单Id
   * @param {*} position   // 刷新位置
   * @param {*} btn        // 执行的按钮
   */
  refreshByButtonResult = (menuId, position, btn, id) => {
    const { config, group, BID } = this.state

    if (group.uuid !== menuId) return

    if ((position === 'mainline' || position === 'popclose') && config.setting.supModule && BID) {
      MKEmitter.emit('reloadData', config.setting.supModule, BID)
    } else if (position === 'grid' && config.wrap.datatype === 'static') {
      if (config.wrap.refocus && group.setting.focus === config.wrap.refocus) {
        let _group = fromJS(group).toJS()
        _group.setting.focus = config.wrap.focus
  
        this.setState({
          group: _group
        })
      }
      this.setState({
        data: null
      }, () => {
        this.setState({data: {$$empty: true}})
      })
    } else {
      this.loadData()
    }

    if (id) {
      MKEmitter.emit('resetSelectLine', config.uuid, id, '')
    }

    this.execSuccess(btn, id)
  }

  resetParentParam = (MenuID, id, data) => {
    const { config } = this.state

    if (!config.setting.supModule || config.setting.supModule !== MenuID) return
    
    if (id !== this.state.BID || id !== '') {
      if (config.wrap.datatype === 'static' || (config.setting.supModule && !id)) {
        this.setState({
          data: null,
          BID: id,
          BData: data
        }, () => {
          this.setState({
            data: {$$empty: true}
          })
        })
      } else {
        this.setState({ BID: id, BData: data }, () => {
          this.loadData()
        })
      }
    }
  }

  execSuccess = (btn, id) => {
    if (btn.linkmenu && btn.linkmenu.length > 0) {
      let menu_id = btn.linkmenu[btn.linkmenu.length - 1]
      let menu = window.GLOB.mkThdMenus.get(menu_id) || ''

      if (!menu) return

      let newtab = {
        ...menu,
        param: {$BID: id || ''}
      }

      MKEmitter.emit('modifyTabs', newtab, true)
    }
  }

  async loadData (refocus) {
    const { config, BID, group } = this.state

    if (config.wrap.refocus && !refocus && group.setting.focus === config.wrap.refocus) {
      let _group = fromJS(group).toJS()
      _group.setting.focus = config.wrap.focus

      this.setState({
        group: _group
      })
    }

    if (config.wrap.datatype === 'public') {
      MKEmitter.emit('reloadData', config.wrap.publicId)
      return
    } else if (config.wrap.datatype === 'static') {
      this.setState({
        data: {$$empty: true}
      })
      return
    } else if (config.setting.supModule && !BID) {
      this.setState({
        data: null
      }, () => {
        this.setState({data: {$$empty: true}})
      })
      MKEmitter.emit('resetSelectLine', config.uuid, '', '')
      return
    }

    let searches = []
    if (config.setting.useMSearch) { // 主表搜索条件
      searches = window.GLOB.SearchBox.get(config.$searchId) || []
    }

    if (config.$s_req && searches.filter(item => item.required && item.value === '').length > 0) {
      return
    }

    this.setState({
      loading: true
    })

    let _orderBy = config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, searches, _orderBy, 1, 1, BID)

    let result = await Api.genericInterface(param)
    if (result.status) {
      let _data = result.data && result.data[0] ? result.data[0] : {$$empty: true}

      _data.$$uuid = _data[config.setting.primaryKey] || ''

      this.setState({
        data: null,
        loading: false
      }, () => {
        this.setState({data: _data})
      })

      MKEmitter.emit('resetSelectLine', config.uuid, _data.$$uuid, _data)

      if (result.message) {
        if (result.ErrCode === 'Y') {
          Modal.success({
            title: result.message
          })
        } else if (result.ErrCode === 'S') {
          notification.success({
            top: 92,
            message: result.message,
            duration: 2
          })
        }
      }
    } else {
      this.setState({
        loading: false,
      })
      
      if (!result.message) return
      if (result.ErrCode === 'N') {
        Modal.error({
          title: result.message,
        })
      } else if (result.ErrCode !== '-2') {
        notification.error({
          top: 92,
          message: result.message,
          duration: 10
        })
      }
    }
  }

  mkFormSubmit = (btnId, callback) => {
    const { group } = this.state

    if (group.uuid !== btnId) return

    this.formRef.handleConfirm().then(res => {
      MKEmitter.emit('triggerFormSubmit', {menuId: btnId, form: res})
    }, () => {
      callback && callback()
    })
  }

  closeTab = () => {
    const { group } = this.state

    MKEmitter.emit('closeTabView', group.subButton.$MenuID)
  }

  resetTab = () => {
    const { data } = this.state

    this.setState({
      data: null
    }, () => {
      this.setState({data: data})
    })
  }

  render() {
    const { config, loading, BID, BData, data, group } = this.state

    if (config.wrap.empty === 'hidden' && (!data || data.$$empty)) return null
    if (config.idCtrl && (!data || data.$$empty)) return null
    
    return (
      <div className="custom-simple-form-box" id={'anchor' + config.uuid} style={{...config.style}}>
        {loading ?
          <div className="loading-mask">
            <div className="ant-spin-blur"></div>
            <Spin />
          </div> : null
        }
        <NormalHeader config={config} />
        {data ? <MutilForm
          BID={BID}
          BData={BData}
          data={data}
          action={group}
          unload={config.setting.supModule && !BID}
          inputSubmit={() => this.mkFormSubmit(group.uuid)}
          wrappedComponentRef={(inst) => this.formRef = inst}
        /> : null}
        {data ? <div className={'mk-form-action ' + (group.$button || '')}>
          <NormalButton
            BID={BID}
            btn={group.subButton}
            setting={config.setting}
            columns={config.columns}
            selectedData={data.$$empty ? [] : [data]}
          />
          {group.resetButton && group.resetButton.enable === 'true' ? <Button type="link" onClick={this.resetTab} style={group.resetButton.style}>{group.resetButton.label}</Button> : null}
          {group.closeButton && group.closeButton.enable === 'true' ? <Button type="link" onClick={this.closeTab} style={group.closeButton.style}>{group.closeButton.label}</Button> : null}
        </div> : null}
      </div>
    )
  }
}

export default SimpleForm